--[[ Manual Cutting
Author:	Ifko[nator] überarbeitet t0xic0m
Datum:	25.08.2015
Bearbeitungs Datum:	10.04.2016

Version:	v1
]]

Harvester = {};

local cuttedTrees = 0;

function Harvester.prerequisitesPresent(specializations) 
    return SpecializationUtil.hasSpecialization(WoodHarvester, specializations);
end; 


function Harvester:load(xmlFile)
	WoodHarvester.onDelimbTree = Utils.overwrittenFunction(WoodHarvester.onDelimbTree, Harvester.NEWonDelimbTree);
	
	self.cutMaxRadius = 1;

    self.cutLengthMin = 1;
    self.cutLengthMax = 18;
    self.cutLengthStep = 1;
end;

function Harvester:delete()
end;

function Harvester:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Harvester:keyEvent(unicode, sym, modifier, isDown)
end;

function Harvester:NEWonDelimbTree(onDelimbTree, state) 
    self.isAttachedSplitShapeMoving = state;
    
	if not state then
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
			self:cutTree(0);
		end;
    end;
end;


function Harvester:update(dt)
	if self:getIsActiveForInput(true) and self:getIsTurnedOn() then
		if self.cutNode ~= nil then
			if self.hasAttachedSplitShape then
				if not self.isAttachedSplitShapeMoving and self:getAnimationTime(self.cutAnimation.name) == 1 then
					if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
						self:cutTree(self.currentCutLength);
					end
				end
				elseif self.curSplitShape ~= nil then
					if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
						self:cutTree(0);
						cuttedTrees = cuttedTrees + 1;
						--print(cuttedTrees)
					end
				end
			end
		end

end;


function Harvester:updateTick(dt)
	if self:getIsActive() then
        self.warnInvalidTree = false;
        self.warnInvalidTreeRadius = false;

        if self:getIsTurnedOn() then
            if self.attachedSplitShape == nil and self.cutNode ~= nil then
                local x,y,z = getWorldTranslation(self.cutNode);
                local nx,ny,nz = localDirectionToWorld(self.cutNode, 1,0,0);
                local yx,yy,yz = localDirectionToWorld(self.cutNode, 0,1,0);

                if self.curSplitShape == nil and (self.cutReleasedComponentJoint == nil or self.cutReleasedComponentJointRotLimitX == 0) then
                    local shape, minY, maxY, minZ, maxZ = findSplitShape(x,y,z, nx,ny,nz, yx,yy,yz, self.cutSizeY, self.cutSizeZ);
                    if shape ~= 0 then
                            local treeDx,treeDy,treeDz = localDirectionToWorld(shape, 0,1,0); -- wood harvester trees always grow in the y direction
                            local cosTreeAngle = Utils.dotProduct(nx,ny,nz, treeDx,treeDy,treeDz);
                            -- Only allow cutting if the cut header is approximately parallel to the tree (70° offset)
                            if cosTreeAngle >= 0.35 then
                                local radius = math.max(maxY-minY, maxZ-minZ)*0.5 * cosTreeAngle;
                                if radius > self.cutMaxRadius then
                                    self.warnInvalidTreeRadius = true;
                                else
                                    self.lastDiameter = math.max(maxY-minY, maxZ-minZ);
                                    self.curSplitShape = shape;
                                end
                            end
                        end
                    end
                               
                if self.curSplitShape ~= nil then
					local minY,maxY, minZ,maxZ = testSplitShape(self.curSplitShape, x,y,z, nx,ny,nz, yx,yy,yz, self.cutSizeY, self.cutSizeZ)
                    if minY == nil then
                        self.curSplitShape = nil;
                    else
                        -- check if cut would be below y=0 (tree CoSy)
                        local cutTooLow = false;
                        local x,y,z = localToLocal(self.cutNode, self.curSplitShape, 0,minY,minZ);
                        cutTooLow = cutTooLow or y < 0.01;
                        local x,y,z = localToLocal(self.cutNode, self.curSplitShape, 0,minY,maxZ);
                        cutTooLow = cutTooLow or y < 0.01;
                        local x,y,z = localToLocal(self.cutNode, self.curSplitShape, 0,maxY,minZ);
                        cutTooLow = cutTooLow or y < 0.01;
                        local x,y,z = localToLocal(self.cutNode, self.curSplitShape, 0,maxY,maxZ);
                        cutTooLow = cutTooLow or y < 0.01;
                        if cutTooLow then
                            self.curSplitShape = nil;
                        end;
                    end
                end
               
                if self.curSplitShape == nil and self.cutTimer > -1 then
                    self:onCutTree(0);
                    if g_server ~= nil then
                        g_server:broadcastEvent(WoodHarvesterOnCutTreeEvent:new(self, 0), nil, nil, self);
                    end;                    
                end;
                
            end
        end

        if self.isServer then
            if self.attachedSplitShape == nil then
               if self.cutReleasedComponentJoint ~= nil and self.cutReleasedComponentJointRotLimitX ~= 0 then
                   self.cutReleasedComponentJointRotLimitX = math.min(self.cutReleasedComponentJointRotLimitX+self.cutReleasedComponentJointRotLimitXSpeed*dt, 0);
                   setJointRotationLimit(self.cutReleasedComponentJoint.jointIndex, 0, true, self.cutReleasedComponentJointRotLimitX, 0);
                end;
                if self.cutReleasedComponentJoint2 ~= nil and self.cutReleasedComponentJoint2RotLimitX ~= 0 then
                    self.cutReleasedComponentJoint2RotLimitX = math.max(self.cutReleasedComponentJoint2RotLimitX-self.cutReleasedComponentJoint2RotLimitXSpeed*dt, 0);
                    setJointRotationLimit(self.cutReleasedComponentJoint2.jointIndex, 0, true, -self.cutReleasedComponentJoint2RotLimitX, self.cutReleasedComponentJoint2RotLimitX);
                end;
            end
        end;
    end
    
    if self.isServer then
        if self.playDelayedGrabAnimationTime ~= nil then
            if self.playDelayedGrabAnimationTime < g_currentMission.time then
                self.playDelayedGrabAnimationTime = nil;
                if self:getAnimationTime(self.grabAnimation.name) > 0 then
                    if self.grabAnimation.name ~= nil and self.attachedSplitShape == nil then
                        self:setAnimationStopTime(self.grabAnimation.name, 0);
                        self:playAnimation(self.grabAnimation.name, -self.grabAnimation.speedScale, self:getAnimationTime(self.grabAnimation.name), false);
                    end;             
                end;
            end;
        end;
    end;
end;


function Harvester:draw()
	if self:getIsTurnedOn() then
		if self:getIsActive() then
			local CuttedTreesString = string.format(Harvester.CUTTED_TREES, cuttedTrees);
			
			renderText(g_currentMission.fillLevelIconOverlay.x + 2*g_currentMission.fillLevelTextOffsetX, g_currentMission.speedHud.y, g_currentMission.fillLevelTextSizeSmall, CuttedTreesString);
		end;
	end;
end;

function Harvester:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		cuttedTrees = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#cuttedTrees"), cuttedTrees);
		
		cuttedTrees = cuttedTrees;
	end;

	return BaseMission.VEHICLE_LOAD_OK;
end;

function Harvester:getSaveAttributesAndNodes(nodeIdent)
    local attributes = 'cuttedTrees="'.. tostring(cuttedTrees) .. '"';

	return attributes, nil;
end;